==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2107_155.zip: ronportal/ron/mvc/jsonoutputobject.php <==
<?php
/**
 * stanislav.elbl, 19.02.2021
 */

namespace Ron\MVC;

use Ron\Http\App;
use Ron\Http\HttpHeaders;
use Ron\Http\HttpResponse;
use Ron\Message;
use Ron\MessageManager;
use Ron\StringUtils;


class JsonOutputObject implements OutputObject
{
    const RESULT_OK = 'OK';
    const RESULT_ERROR = 'error';

    private $data;
    /** @var bool|string */
    public $compress = false; // true/false/gzip - pozor gzip funguje jen v IE, nepodarilo se rozchodit v FF ani GCH
    public $fileName = null;

    /**
     * @param array|null $data
     */
    public function __construct($data = null)
    {
        if ( isset($data) ) {
            $this->data = $data;
        } else {
            $this->data = ['result' => self::RESULT_OK];
        }
    }


    /**
     * @param string $name
     * @param mixed $value
     * @return JsonOutputObject
     */
    public function addData($name, $value)
    {
        $this ->data[$name] = $value;
        return $this;
    }

    /**
     * @param string $name
     * @param string $text
     * @param bool   $htmlConvert
     * @return JsonOutputObject
     */
    public function addText($name, $text, $htmlConvert = false)
    {
        $this ->data[$name] = isset($text) ? StringUtils::jsonText($text, $htmlConvert) : null;
        return $this;
    }


    private function createError($errorText, $errorField = null, $errorDescription = null) {
        $result = [
            'errorText' => StringUtils::jsonText($errorText . ($errorDescription ? ' ' . $errorDescription : ''))
        ];

        if ($errorField) $result['errorField'] = $errorField;
        if ($errorDescription) $result['errorDescription'] = StringUtils::jsonText($errorDescription);

        return $result;
    }

    /**
     * @param string|\Exception $errorText
     * @param string $errorField
     * @param string $errorDescription
     * @return JsonOutputObject
     */
    public function addError($errorText, $errorField = null, $errorDescription = null)
    {
        if ( !isset($this ->data['errors']) ) {
            $this ->data['errors'] = [];
        }

        if ($errorText instanceof \Exception) {
            $errorText = $errorText ->getMessage();
        }

        $this ->data['errors'][] = $this ->createError($errorText, $errorField, $errorDescription);
        $this ->data['result'] = self::RESULT_ERROR;

        return $this;
    }


    /**
     * @param string $message
     * @return JsonOutputObject
     */
    public function addDebugMessage($message)
    {
        if (App::getDebugMode()) {
            if (!isset($data['debugMessages'])) {
                $this ->data['debugMessages'] = [];
            }

            $this ->data['debugMessages'][] = StringUtils::jsonText($message);
        }

        return $this;
    }


    /**
     * @param string $message
     * @return JsonOutputObject
     */
    public function addLogMessage($message)
    {
        if (!isset($data['logMessages'])) {
            $this ->data['logMessages'] = [];
        }

        $this ->data['logMessages'][] = StringUtils::jsonText($message);

        return $this;
    }


    /**
     * @param string $result
     * @return JsonOutputObject
     */
    public function setResult($result)
    {
        $this->data['result'] = $result;

        return $this;
    }


    /**
     * @param MessageManager $messageManager
     * @return JsonOutputObject
     */
    public function addMessages($messageManager)
    {
        foreach ($messageManager ->getMessages() as $message) {
            switch ($message ->type) {
                case Message::ERROR :
                    $this ->addError($message ->text, $message ->fieldName, $message ->description);
                    break;

                case Message::DEBUG :
                    $this ->addDebugMessage($message ->text);
                    break;

                case Message::LOG :
                    $this ->addLogMessage($message ->text);
                    break;
            }
        }

        return $this;
    }


    /**
     * @return void
     */
    public function output()
    {
        // TODO: otestovat, jestli "gzip" funguje i v Androidu
        if ($this ->compress === 'gzip') { // POZOR! funguje jen v IE!
            HttpResponse::setHeader(HttpHeaders::CONTENT_TYPE, 'application/json; charset=utf-8');
            HttpResponse::setHeader(HttpHeaders::CONTENT_ENCODING, 'gzip');
            HttpResponse::renderHeaders();

            print gzencode(json_encode($this->data)); // jako gzcompress, ale pridava i header
        } else if ($this ->compress) {
            header('Content-Type: application/x-download');
            //header('Content-Encoding: gzip'); - nefunguje
            header('Cache-Control: no-cache, no-store, max-age=0, must-revalidate');
            header('Pragma: no-cache');

            if ($this ->fileName) {
                header('Content-Disposition: attachment; filename="' . $this ->fileName . '"');
            }

            HttpResponse::renderHeaders();

            print gzcompress(json_encode($this->data));
        } else {
            HttpResponse::setHeader('Content-Type', 'application/json; charset=utf-8');
            HttpResponse::renderHeaders();

            echo json_encode($this->data);
        }
    }

}

==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2206_162.zip: ronportal/ron/mvc/jsonoutputobject.php <==
<?php
/**
 * stanislav.elbl, 19.02.2021
 */

namespace Ron\MVC;

use Ron\DB\DBException;
use Ron\Http\App;
use Ron\Http\HttpHeaders;
use Ron\Http\HttpResponse;
use Ron\Message;
use Ron\MessageManager;
use Ron\RonException;
use Ron\StringUtils;


class JsonOutputObject implements OutputObject
{
    const RESULT_OK = 'OK';
    const RESULT_ERROR = 'error';

    private $data;
    /** @var bool|string */
    public $compress = false; // true/false/gzip - pozor gzip funguje jen v IE, nepodarilo se rozchodit v FF ani GCH
    public $fileName = null;

    /**
     * @param array|null $data
     */
    public function __construct($data = null)
    {
        if ( isset($data) ) {
            $this->data = $data;
        } else {
            $this->data = ['result' => self::RESULT_OK];
        }
    }


    /**
     * @param string $name
     * @param mixed $value
     * @return JsonOutputObject
     */
    public function addData($name, $value)
    {
        $this ->data[$name] = $value;
        return $this;
    }

    /**
     * @param string $name
     * @param string $text
     * @param bool   $htmlConvert
     * @return JsonOutputObject
     */
    public function addText($name, $text, $htmlConvert = false)
    {
        $this ->data[$name] = isset($text)
            ? StringUtils::jsonText($text, $htmlConvert)
            : null;

        return $this;
    }


    private function createError($errorText, $errorField = null, $errorDescription = null) {
        $result = [
            'errorText' => StringUtils::jsonText($errorText . ($errorDescription ? ' ' . $errorDescription : ''))
        ];

        if ($errorField) {
            $result['errorField'] = $errorField;
        }

        if ($errorDescription) {
            $result['errorDescription'] = StringUtils::jsonText($errorDescription);
        }

        return $result;
    }

    /**
     * @param string|\Exception $errorText
     * @param string $errorField
     * @param string $errorDescription
     * @return JsonOutputObject
     */
    public function addError($errorText, $errorField = null, $errorDescription = null)
    {
        if ( !isset($this ->data['errors']) ) {
            $this ->data['errors'] = [];
        }

        global $development;

        if (($development || App::getDebugMode()) && $errorText instanceof DBException) {
            $errorDescription = $errorDescription ?: $errorText->sql;
            $errorField = $errorField ?: $errorText->getFieldName();
            $errorText = $errorText->getMessage() . " \n" . $errorText->dbMessage;
        } else if ($errorText instanceof RonException) {
            $errorDescription = $errorDescription ?: $errorText ->getErrorDescription();
            $errorField = $errorField ?: $errorText ->getFieldName();
            $errorText = $errorText->getMessage();
        } else if ($errorText instanceof \Exception) {
            $errorText = $errorText ->getMessage();
        }

        $this ->data['errors'][] = $this ->createError($errorText, $errorField, $errorDescription);
        $this ->data['result'] = self::RESULT_ERROR;

        return $this;
    }


    /**
     * @param string $message
     * @return JsonOutputObject
     */
    public function addDebugMessage($message)
    {
        if (App::getDebugMode()) {
            if (!isset($data['debugMessages'])) {
                $this ->data['debugMessages'] = [];
            }

            $this ->data['debugMessages'][] = StringUtils::jsonText($message);
        }

        return $this;
    }


    /**
     * @param string $message
     * @return JsonOutputObject
     */
    public function addLogMessage($message)
    {
        if (!isset($data['logMessages'])) {
            $this ->data['logMessages'] = [];
        }

        $this ->data['logMessages'][] = StringUtils::jsonText($message);

        return $this;
    }


    /**
     * @param string $result
     * @return JsonOutputObject
     */
    public function setResult($result)
    {
        $this->data['result'] = $result;

        return $this;
    }


    /**
     * @param MessageManager $messageManager
     * @return JsonOutputObject
     */
    public function addMessages($messageManager)
    {
        foreach ($messageManager ->getMessages() as $message) {
            switch ($message ->type) {
                case Message::ERROR :
                    $this ->addError($message ->text, $message ->fieldName, $message ->description);
                    break;

                case Message::DEBUG :
                    $this ->addDebugMessage($message ->text);
                    break;

                case Message::LOG :
                    $this ->addLogMessage($message ->text);
                    break;
            }
        }

        return $this;
    }


    /**
     * @return void
     */
    public function output()
    {
        // TODO: otestovat, jestli "gzip" funguje i v Androidu
        if ($this ->compress === 'gzip') { // POZOR! funguje jen v IE!
            HttpResponse::setHeader(HttpHeaders::CONTENT_TYPE, 'application/json; charset=utf-8');
            HttpResponse::setHeader(HttpHeaders::CONTENT_ENCODING, 'gzip');
            HttpResponse::renderHeaders();

            print gzencode(json_encode($this->data)); // jako gzcompress, ale pridava i header
        } else if ($this ->compress) {
            header('Content-Type: application/x-download');
            //header('Content-Encoding: gzip'); - nefunguje
            header('Cache-Control: no-cache, no-store, max-age=0, must-revalidate');
            header('Pragma: no-cache');

            if ($this ->fileName) {
                header('Content-Disposition: attachment; filename="' . $this ->fileName . '"');
            }

            HttpResponse::renderHeaders();

            print gzcompress(json_encode($this->data));
        } else {
            HttpResponse::setHeader('Content-Type', 'application/json; charset=utf-8');
            HttpResponse::renderHeaders();

            echo json_encode($this->data);
        }
    }

}
